
%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "globals.h"
#include "sym.h"

char *pullstring();

#define COMMENT_CHAR '#'

#define YYDEBUG 1

%}

%union {
	symbol *sym;
	int     reg;
}

%type	<sym>	variable
%token	<sym>	IDENTIFIER

%type	<reg>	condition
%type	<reg>	expr
%type	<reg>	arglist paramlist

%type	<reg>	statement
%token	<reg>	INCLUDE DEFINE
%token	<reg>	CHAR
%token	<reg>	RETURN EXIT GOTO
%token	<reg>	IF ELSE WHILE DO FOR SWITCH CASE DEFAULT BREAK

%token	<reg>	STRING
%token	<reg>	NUMBER

%token	<reg>	LEXERR

%right		'='
%left		yAND yOR
%nonassoc	'<' '>' LTE GTE yEQU yNEQ
%left		'+' '-'
%left		'*' '/' '%'
%left		yINCR yDECR
%left		'['
%left		'!' '~' UPLUS UMINUS

%start	program

%%

program	: ablk
		| program ablk
		;

ablk	: IDENTIFIER '(' paramlist ')' '{' {
			addsym($1->name, 'F');
		} decls {
			enter_function($1);
		} statements '}' {
			leave_function($1);
			kill_scope();
			param_clear();
		}
		;

decls	: /* nothing */ {
		}
		| decls CHAR IDENTIFIER ';' {
			local_ref($3->name, 1);
		}
		;

blkstatement
		: '{' statements '}' {
		}
		;

statements
		: /* nothing */ {
			yyerrok;
		}
		| statements labels astatement {
			yyerrok;
		}
		;

labels	: /* nothing */ {
		}
		| labels IDENTIFIER ':' {
			addsym($2->name, 'l');
			printf("L%d:\n", $2->uniq);
		}
		| labels CASE NUMBER ':' {
			add_case($3);
		}
		| labels DEFAULT ':' {
			add_default();
		}
		;

astatement
		: blkstatement {
		}
		| statement ';' {
		}
		/*
		 *  BREAK from SWITCH or FOR/WHILE
		 */
		| BREAK ';' {
			break_loop();
		}
		/*
		 *  SWITCH block
		 *
		 *  Accumulate expr
		 *  Jump over switch block
		 *  Select case/default
		 *  Jump back.
		 */
		| SWITCH '(' expr ')' {
			start_switch();
		} blkstatement {
			end_switch();
		}
		/*
		 *  Single IF
		 */
		| IF '(' condition ')' astatement {
			printf("L%d: # not condition\n", $3);
		}
		/*
		 *  Dualheaded IF
		 */
		| IF '(' condition ')' astatement ELSE {
			printf("JMP L%d\n", $<reg>$ = uniq++);
			printf("L%d: # else block\n", $3);
		} astatement {
			printf("L%d: # end if\n", $<reg>7);
		}
		/*
		 *  DO loop
		 */
		| DO {
			$<reg>$ = start_loop();
		} astatement WHILE '(' expr ')' ';' {
			printf("JNZ L%d # do-loop top\n", $<reg>2);
			end_loop();
		}
		/*
		 *  WHILE loop
		 */
		| WHILE {
			$<reg>$ = start_loop();
		} '(' condition ')' astatement {
			printf("JMP L%d # to while check\n", $<reg>2);
			printf("L%d: # end while\n", $4);
			end_loop();
		}
		/*
		 * FOR creates a mesh of jumps.
		 * If condition is blank, JZ 1f is skipped with a jump.
		 *
		 *    statement  (entry)
		 * 3: statement  (condition, true if blank)
		 *    JZ 1f
		 *    JMP 2f
		 * 4: statement  (trailer code)
		 *    JMP 3b
		 * 2: astatement (loop code)
		 *    JMP 4b
		 * 1:
		 */
		| FOR '(' statement ';' {
			$<reg>$ = uniq;			/* $5 */
			uniq += 10;
			printf("L%d: # 2nd for-stmt\n", $<reg>$ + 1);
		} statement ';' {
			if ($6 == 0)
				printf("JMP L%d\n", $<reg>5 + 3);
			else
				printf("JNZ L%d\n", $<reg>5 + 3);
			printf("JMP L%d\n", $<reg>5 + 4);
			printf("L%d: # 3rd for-stmt \n", $<reg>5 + 2);
		} statement ')' {
			printf("JMP L%d\n", $<reg>5 + 1);
			printf("L%d: # for statements\n", $<reg>5 + 3);
			start_loop();
		} astatement {
			if ($9)
				printf("JMP L%d\n", $<reg>5 + 2);
			else
				printf("JMP L%d\n", $<reg>5 + 1);
			printf("L%d:\n", $<reg>5 + 4);
			end_loop();
		}
		/*
		 * disgusting, wasn't it ?
		 */
		| error {
			error(NULL);
		}
		;

condition
		: expr {
			printf("JZ L%d # to not-condition\n", $$ = uniq++);
		}
		;

statement
		: /* nothing */ {
			$$ = 0;
		}
		| GOTO IDENTIFIER {
			printf("JMP L%d # label %s\n", $2->uniq, $2->name);
			$$ = 1;
		}
		| RETURN expr {
			return_jump();
			$$ = 1;
		}
		| RETURN {
			return_jump();
			$$ = 1;
		}
		| expr {
			$$ = 1;
		}
		;

arglist	
		: /* nothing */ {
			$$ = 0;
		}
		| expr {
			printf("PUSH # arglist\n");
			$$ = 1;
		}
		| arglist ',' expr {
			printf("PUSH # arglist\n");
			$$ = 1 + $1;
		}
		;

paramlist
		: /* nothing */ {
			$$ = 0;
		}
		| IDENTIFIER {
			param_ref($1->name);
			$$ = 1;
		}
		| IDENTIFIER ',' paramlist {
			param_ref($1->name);
			$$ = 1 + $3;
		}
		;

variable
		: IDENTIFIER {
			var_ref($1);
			$$ = $1;
		}
		;

expr	: NUMBER {
			printf("LDI %d # number\n", $1);
		}
		| STRING {
			printf("LDI L%d # string\n", $1);
		}
		| variable {
			op_load($1);
		}
		| IDENTIFIER '(' arglist ')' {
			fcall($1, $3);
		}
		| '(' expr ')' {
			;
		}
		| expr '[' {
			printf("PUSH\n");
		} expr ']' {
			printf("ADD\n");
			printf("XLAT\n");
		}
		| '+' expr %prec UPLUS {
			;
		}
		| '-' expr %prec UMINUS {
			printf("NEG\n");
		}
		| '!' expr {
			printf("NOT\n");
		}
		| expr yEQU {
			printf("PUSH\n");
		} expr {
			printf("CMP\n");
		}
		| expr yNEQ {
			printf("PUSH\n");
		} expr {
			printf("CMP\n");
			printf("NOT\n");
		}
		| expr yAND {
			printf("PUSH\n");
		} expr {
			printf("ANDL\n");
		}
		| expr yOR {
			printf("PUSH\n");
		} expr {
			printf("ORL\n");
		}
		| expr '*' {
			printf("PUSH\n");
		} expr {
			printf("MUL\n");
		}
		| expr '/' {
			printf("PUSH\n");
		} expr {
			printf("DIV\n");
		}
		| expr '%' {
			printf("PUSH\n");
		} expr {
			printf("MOD\n");
		}
		| expr '+' {
			printf("PUSH\n");
		} expr {
			printf("ADD\n");
		}
		| expr '-' {
			printf("PUSH\n");
		} expr {
			printf("SUB\n");
		}
		| expr '<' {
			printf("PUSH\n");
		} expr {
			printf("TEST_LT\n");
		}
		| expr LTE {
			printf("PUSH\n");
		} expr {
			printf("TEST_LTE\n");
		}
		| expr '>' {
			printf("PUSH\n");
		} expr {
			printf("TEST_GT\n");
		}
		| expr GTE {
			printf("PUSH\n");
		} expr {
			printf("TEST_GTE\n");
		}
		| variable '=' expr {
			op_store($1);
		}
		| variable yINCR {
			op_postinc($1);
		}
		| variable yDECR {
			op_postdec($1);
		}
		| yINCR variable {
			op_preinc($2);
		}
		| yDECR variable {
			op_predec($2);
		}
		;

%%

static struct {
  char *name;
  int   token;
} keywords[] = {
  { "goto",		GOTO,	},
  { "return",	RETURN,	},
  { "if",		IF,	},
  { "else",		ELSE,	},
  { "for",		FOR,	},
  { "while",	WHILE,	},
  { "do",		DO,	},
  { "switch",	SWITCH,	},
  { "case",		CASE,	},
  { "default",	DEFAULT,},
  { "break",	BREAK,	},
  { "char",		CHAR,	},
};

yyerror(s)
	char *s;
{
	error("%s", s);
}

char *
pullstring()
{
	char str[1024];
	char *cp = str;
	int c, i;

#define c2i(c) (isdigit(c) ? ((c) - '0') : (10 + toupper(c) - 'A'))

	while ((c = getch()) != '"') {
		if (c == -1)
			error("EOF in string");
		if (c == '\\') {
			switch (c = getch()) {
			case '"':
				c = '"';
				break;
			case 'r':
				c = '\r';
				break;
			case 'n':
				c = '\n';
				break;
			case 'b':
				c = '\b';
				break;
			case 't':
				c = '\t';
				break;
			case '0': case '1': case '2': case '3':
			case '4': case '5': case '6': case '7':
				i = c2i(c);
				c = getch();
				if (c >= '0' && c < '8') {
					i = ((i << 3) | c2i(c));
					c = getch();
					if (c >= '0' && c < '8')
						i = ((i << 3) | c2i(c));
					else
						ungetch(c);
				} else
					ungetch(c);
				c = i;
				break;
			case 'x':
				i = 0;
				c = getch();
				if (isxdigit(c)) {
					i = ((i << 4) | c2i(c));
					c = getch();
					if (isxdigit(c))
						i = ((i << 4) | c2i(c));
					else
						ungetch(c);
				} else
					ungetch(c);
				c = i;
				break;
			default:
				error("unknown escape %c", c);
			}
		}
		if (c == 0) {
			error("zero in string");
		}
		*cp++ = c;
	}
	*cp++ = 0;
	return (strdup(str));
}

pullchar()
{
	int c, i = 0;

	while ((c = getch()) != '\'') {
		if (c == -1)
			error("EOF in character");
		if (c == '\\') {
			switch (c = getch()) {
			case 'r':
				c = '\r';
				break;
			case 'n':
				c = '\n';
				break;
			case 'b':
				c = '\b';
				break;
			case 't':
				c = '\t';
				break;
			default:
				error("unknown escape %c", c);
			}
		}
		i = ((i << 8) | (c & 0xff));
	}
	return (i);
}

pullnumber(c)
	int c;
{
	char buf[512];
	char *cp = buf;
	int x;

	do {
		*cp++ = c;
	} while ((c = getch()), (c == 'x' || isxdigit(c)));
	ungetch(c);

	*cp++ = 0;
	if (sscanf(buf, "%i", &x) != 1)
		error("invalid number: %s", buf);

	return (x);
}

yylex()
{
	int token, value, c, next, i;

	yylval.reg = 0;

	for (;;) {
		while ((c = getch()) == ' ' || c == '\t' || c == '\n')
			;
		if (c == -1)
			return (0);

		if (c != COMMENT_CHAR)
			break;

		while ((c = getch()) != '\n' && c != -1)
			;
		if (c == -1)
			return (0);
	}
	switch (c) {
	case '(': case ')':
	case '{': case '}':
	case '[': case ']':
	case ':': case ';': case ',': case '.':
	case '*': case '/':
	case '%': case '^': case '~':
	case '#':
		return (c);

	case '+':
		if ((next = getch()) == '+')
			return (yINCR);
		ungetch(next);
		return (c);
	case '-':
		if ((next = getch()) == '-')
			return (yDECR);
		ungetch(next);
		return (c);

	case '&':
		if ((next = getch()) == '&')
			return (yAND);
		ungetch(next);
		return (c);
	case '|':
		if ((next = getch()) == '|')
			return (yOR);
		ungetch(next);
		return (c);
	case '>':
		if ((next = getch()) == '=')
			return (GTE);
		ungetch(next);
		return (c);
	case '<':
		if ((next = getch()) == '=')
			return (LTE);
		ungetch(next);
		return (c);
	case '=':
		if ((next = getch()) == '=')
			return (yEQU);
		ungetch(next);
		return (c);
	case '!':
		if ((next = getch()) == '=')
			return (yNEQ);
		ungetch(next);
		return (c);

	case '"':
		yylval.reg = pure_string(pullstring());
		return (STRING);

	case '\'':
		yylval.reg = pullchar();
		return (NUMBER);

	case '0': case '1': case '2': case '3': case '4':
	case '5': case '6': case '7': case '8': case '9':
		yylval.reg = pullnumber(c);
		return (NUMBER);

	default: {
		char buffer[512];
		char *cp = buffer;
		symbol *sym;

		do {
			*cp++ = c;
			c = getch();
		} while (isalnum(c) || c == '_');
		ungetch(c);
		*cp = 0;

		for (i = 0; i < __SIZEOF(keywords); ++i)
			if (!strcmp(buffer, keywords[i].name))
				return (keywords[i].token);

		if ((sym = symlookup(buffer)) == NULL)
			sym = addsym(buffer, 0);

		yylval.sym = sym;
		return (IDENTIFIER);
		}
	}
}

static char  unputb[512];
static char *unputp = unputb;

ungetch(c)
	int c;
{
	*unputp++ = c;
}

getch()
{
	if (unputp != unputb)
		return (*--unputp);

	return (getchar());
}

codeparse()
{
	kill_scope();
	param_clear();

	if (yyparse())
		error("oops");

	return (0);
}

