!----------------------------------------------------------------------
!
!  Bastardized Mocoma ARP-150.
!
!----------------------------------------------------------------------
!
! P1.7 TX      (low pin transmits)
! P1.6 SHIFT   (low pin enables TX VCO)
! P1.5 PSAVE   (low pin saves power)
! P1.4 MONI    (low pin enables audio)
! P1.3 AF TX   (1750Hz square wave)
! P1.2 STB     (synth)
! P1.1 DATA    (synth)
! P1.0 CLK     (synth)
!
! P3.7 SQ      (input)
! P3.6 LOCK    (input)
! P3.5 LAMP    (grounding button, yellow)
! P3.4 PTT     (grounding button, T0 wakeup)
! P3.3 DOWN    (grounding button, screw, INT1 wakeup)
! P3.2 UP      (grounding button, screw, INT0 wakeup)
! P3.1 BEEP    (grounding button, "HI/LOW" button)
! P3.0 SIMPLEX (grounding button, enable duplex/reverse, simplex high)
!
! A15  COL0    (high pin grounds common cathode)
! A14  COL1    (high pin grounds common cathode)
! A13  COL2    (high pin grounds common cathode)
!
! A12  SER     (74HC595)
! A11  CLK     (74HC595, both SCLK and RCLK)
!
!----------------------------------------------------------------------

#define TX      P1.7
#define RXVCO   P1.6
#define PSAVE   P1.5
#define MONI    P1.4
#define AFTX    P1.3
#define STB     P1.2
#define DATA    P1.1
#define CLK     P1.0

#define SQ      P3.7
#define LOCK    P3.6
#define LAMP    P3.5
#define PTT     P3.4
#define DOWN    P3.3
#define UP      P3.2
#define BEEP    P3.1
#define SIMPLEX P3.0

#define COL0 0x8000
#define COL1 0x4000
#define COL2 0x2000

#define DIMLEDS ljmp .+3
#define LITLED0 ljmp .+3+COL0
#define LITLED1 ljmp .+3+COL1
#define LITLED2 ljmp .+3+COL2

#define SER  0x1000
#define SCLK 0x0800

#define Rchannel R7
#define Rled     R6
#define Rpsave   R5

#define TICKFLAG  0x20.0
#define UPFLAG    0x20.1
#define DNFLAG    0x20.2
#define LAMPFLAG  0x20.3
#define PSAVING   0x20.4

#define led1kHz     0x30
#define led10kHz    0x31
#define led100kHz   0x32

#define Nreg        0x33
#define Areg        0x34

#define tick        0x35
#define period      0x36

#define ACTIVE 100

#define Stack       0x40

	.org 0x0000
	ajmp start

	.org 0x0003		! INT0
	set UPFLAG
	reti

	.org 0x000B		! Counter 0 overflow, wake up from PTT
	reti

	.org 0x0013		! INT1
	set DNFLAG
	reti

	.org 0x001B		! Timer 1 overflow, wake up for led flipping
	set TICKFLAG
	reti

	.asciz "@(#)ARP150+8031+74595 1.1 970809"

led_byte:

	mov R2, $ 9		! 8+1 for last RCLK
	clr EA
3:
	rlc A			! MSbits
	jc 1f
	ljmp .+3+0*SER
	ljmp .+3+0*SER+SCLK
	ljmp .+3+0*SER
	sjmp 2f
1:
	ljmp .+3+1*SER
	ljmp .+3+1*SER+SCLK
	ljmp .+3+1*SER
2:
	djnz R2, 3b
	set EA
	ret

synth_bits:
1:
	rlc A			! MSbits
	mov DATA, C		! Data
	set CLK
	clr CLK
	djnz R2, 1b		! R2 MSbits
	ret

!   21400: 53,32
!  145000: RX 53,0, TX 106,32

synth_update:

	clr PSAVING
	set PSAVE
	mov period, $ ACTIVE

	mov A, Rchannel	! 0...39

	cjne A, $ 24, 1f	! 145600
1:					! Carry if channel cannot have repeater difference
	orl C, SIMPLEX	! Carry if switch not in duplex/reverse position
	jc 1f			! Skip if "simplex"
	mov C, RXVCO	! Clear if TXVCO SHIFT on ...
	anl C, LAMP		! ... or MONI pressed
	jc 1f			! Skip if "at output
	add A, $ -24	! Channels below "currfreq"
1:
	mov B, A		! remember for readout
	mov R2, A		! and for calculation

	mov Nreg, $ 106
	mov Areg, $ 32		! TX base
	jnb RXVCO, 1f
	mov Nreg, $ 53
	mov Areg, $ 0		! RX base
1:
	jz 3f
	mov A, Areg
2:
	add A, $ 4		! 25 kHz
	anl A, $ 63
	jnz 1f
	inc Nreg		! overflow from 64/65
1:
	djnz R2, 2b
	mov Areg, A
3:

	mov A, $ 0x40
	mov R2, $ 2
	acall synth_bits		! 10 bit N, always 01xxxxxxxx

	mov A, Nreg
	mov R2, $ 8
	acall synth_bits		! rest of N, 8 bits

	mov A, Areg
	rl A					! drop bit 7
	mov R2, $ 7
	acall synth_bits		! 7 bit A

	set STB
	clr STB

	mov A, B
	acall map1kHz
	mov led1kHz, A

	mov A, B
	acall map10kHz
	mov led10kHz, A

	mov A, B
	acall map100kHz
	mov led100kHz, A

	ret

#define MASK0 0b11011110
#define MASK1 0b00010010
#define MASK2 0b01111100
#define MASK3 0b01110110
#define MASK4 0b10110010
#define MASK5 0b11100110
#define MASK6 0b11101110
#define MASK7 0b01010010
#define MASK8 0b11111110
#define MASK9 0b11110110

map1kHz:
	anl A, $ 1
	inc A
	movc A, @A + PC
	ret
	.byte MASK0, MASK5

map10kHz:
	anl A, $ 3
	inc A
	movc A, @A + PC
	ret
	.byte MASK0, MASK2, MASK5, MASK7

map100kHz:
	rr A
	rr A
	anl A, $ 0xF
	inc A
	movc A, @A + PC
	ret
	.byte MASK0, MASK1, MASK2, MASK3, MASK4
	.byte MASK5, MASK6, MASK7, MASK8, MASK9

start:

	clr EA
	mov SP, $ Stack

	mov R0, $ 0xFF
1:
	mov @R0, $ 0
	djnz R0, 1b

	!
	!  Timer 0 - mode 2, 8 bit autoreload counter, PTT wakeup
	!  Timer 1 - mode 2, 8 bit autoreload timer, led flipping
	!
	mov TMOD, $ 0b00100110

	clr TR0
	mov TH0, $ 0xFF
	mov TL0, $ 0xFF
	clr TF0

	clr TR1
	mov TH1, $ 0
	mov TL1, $ 0
	clr TF1

	set IT0
	set IT1			! Falling edge ints from screw

	set EX0
	set EX1			! Wakeup from screw

	set ET0			! Wakeup from Timer 0, PTT
	set ET1			! Wakeup clock from Timer 1

	clr CLK
	clr STB			! idle low outputs

	mov Rchannel, $ 20

	acall synth_update

	set TR0			! Quick wakeup from PTT.
	set TR1			! Keep updating leds.

	set EA			! Enable enabled ones

idle:	!------------------------------------------------------------

	clr TICKFLAG
	mov PCON, $ 1	! IDL

	mov C, LAMP		! "MONI" not pressed, high
	anl C, /SQ		! SQ low closed
	orl C, PSAVING	! PSAVING, no audio
	mov MONI, C		! set moni-output to "close audio" if both

	jc 1f			! Clear iff not saving and squelch open (forced or real)
	clr PSAVING
	set PSAVE
	mov period, $ ACTIVE
1:

	jnb TICKFLAG, 9f
	djnz tick, 9f
	mov tick, $ 110
	djnz period, 9f
	jnb PSAVE, 2f
	jnb SQ, 1f		! /PSAVE was not active for this period, look SQ.
	clr PSAVING
	set PSAVE
	mov period, $ ACTIVE
	sjmp 9f
1:
	set PSAVING
	clr PSAVE
	mov period, $ 10
	sjmp 9f
2:
	set PSAVING
	set PSAVE		! saved last period, now listen for a small while
	mov period, $ 3
9:

	jnb TICKFLAG, 9f
	inc Rled

	cjne Rled, $ 1, 1f
	mov A, led1kHz
	mov C, SQ
	mov ACC.0, C	! Last decimal point as "BUSY"
	lcall led_byte
	ljmp 9f + COL2	! 1 kHz
1:
	cjne Rled, $ 3, 1f
	mov A, led10kHz
	mov C, LOCK
	cpl C
	mov ACC.0, C	! Middle decimal point as "UNLOCK"
	lcall led_byte
	ljmp 9f + COL1	! 10 kHz
1:
	cjne Rled, $ 5, 1f
	mov A, led100kHz
	mov C, PSAVE
	cpl C
	mov ACC.0, C	! First decimal flickers with /PSAVE
	lcall led_byte
	ljmp 9f + COL0	! 100 kHz
1:
	DIMLEDS
	cjne Rled, $ 7, 9f		! each led on 1/8 times, total 3/7, dim on psave
	mov Rled, $ 0

9:	!------------------------------------------------------------

	jnb UPFLAG, 9f
	dec Rchannel
	cjne Rchannel, $ 0xFF, 1f
	mov Rchannel, $ 39
1:
	acall synth_update
	clr UPFLAG
9:
	jnb DNFLAG, 9f
	inc Rchannel
	cjne Rchannel, $ 40, 1f
	mov Rchannel, $ 0
1:
	acall synth_update
	clr DNFLAG

9:	!------------------------------------------------------------

	jb LAMP, 1f			! go if not now pressed
	jb LAMPFLAG, 9f		! go if was already pressed
	set LAMPFLAG
	acall synth_update	! check rptr input
	sjmp 9f
1:
	jnb LAMPFLAG, 9f	! go if was not pressed
	clr LAMPFLAG
	acall synth_update	! return back to rptr output

9:	!------------------------------------------------------------

	jnb PTT, 9f		! if PTT pressed
	jnb BEEP, 9f	! if BEEP pressed

	ajmp idle		!---------------------------------------------
9:

	set MONI		! Close audio for sure
	clr RXVCO		! Shift VCO to TX
	acall synth_update

	mov A, led100kHz
	lcall led_byte
	LITLED0

	clr EA
	clr TX			! TX ON

	jnb BEEP, dobeep
1:
	jb PTT, endtx	! if PTT released
	jb LOCK, 2f
	DIMLEDS
	set TX
	sjmp 1b
2:
	LITLED0
	clr TX
	sjmp 1b

dobeep:
1:
	cpl AFTX		! 1
	nop				! 1
	mov R2, $ 41	! 2
	djnz R2, .		! 82
	jnb BEEP, 1b	! 2 wait BEEP
					! 88 with 3.6864 xtal gives 1745 Hz

endtx:
	set AFTX		! Keep AFTX pin high idle

	set TX			! TX OFF
	set RXVCO		! Shift VCO to RX
	set EA
	acall synth_update

	ajmp idle

