
#define VERSION "2.0 980210"

/*
 *  2.0  980210 CD60 From Mx5x.
 *
 *  Juha, OH5NXO
 */

Rint = 1
Rstk = 2
Rpgm = 3
Rcal = 4	# SCRT "call"
Rret = 5	# SCRT "return"
Rarg = 6	# SCRT "temporary & arguments"
Rbp  = 7	# Extended SCRT, for stack frame

Rtmp  = 8
Rtmp3 = 9
Rram  = 0xA
Rptr  = 0xB
Rtmp2 = 0xC
Rtbl  = 0xD	# Always to tables

ASY = 1
MDM = 2
REG = 4

!----------------------------------------------------------------------
!
!  MAS7205  NMT modem hybrid
!
!  Status register, read A5 high

MDMHOOK   = 0x80 ! /HOOK state

MDMFFSKC  = 0x40 ! RX FFSK data in realtime
MDMINT    = 0x20 ! 100Hz systick
MDMTBE    = 0x04 ! TX buffer empty
MDMDAV    = 0x01 ! RX data available

!  Control register, write A5 high

MDMINTACK = 0x40 ! Acknowledge systick interrupt
MDMTXEN   = 0x02 ! Enable FFSK transmission

MDMINTEN  = 0x11 ! Just and only guessing this one...

!----------------------------------------------------------------------

#define shr4 shr; shr; shr; shr
#define shl4 shl; shl; shl; shl

#define LDREG(r,a)  ldi LO(a) ; plo r ; ldi HI(a) ; phi r
#define ADJUST(r,a) ldi LO(a) ; plo r

#define CALL(f) sep Rcal ; .byte LO(f), HI(f)
#define RETURN  sep Rret

#define BYTE   : .rs 1
#define BUF(n) : .rs n

#define DOSUB(r, d) glo r; smi LO(d); plo r; ghi r; smbi HI(d); phi r
#define DOADD(r, d) glo r; adi LO(d); plo r; ghi r; adci HI(d); phi r

#define PACK(a,b) (((a) << 8) | (b))
#define REGUPD str Rram ; sex Rram ; out REG ; dec Rram

#define SET(x)  ADJUST(Rram,(x)>>8); ldn Rram ; ori  ((x)&0xF) ; REGUPD
#define CLR(x)  ADJUST(Rram,(x)>>8); ldn Rram ; ani ~((x)&0xF) ; REGUPD
#define FLIP(x) ADJUST(Rram,(x)>>8); ldn Rram ; xri  ((x)&0xF) ; REGUPD

#define PULSE(x) SET(x); CLR(x)

#define LIT(BIT) ADJUST(Rram, indicators); ldn Rram; ori  MASK(BIT); str Rram
#define DIM(BIT) ADJUST(Rram, indicators); ldn Rram; ani ~MASK(BIT); str Rram

#define MASK(bit) (1 << (bit))

	.data

!----------------------------------------------------------------------

#define RXE    PACK(output_0, 1)
#define XM     PACK(output_0, 2)
#define RSE    PACK(output_0, 4)

#define SV     PACK(output_1, 1)
#define PL2    PACK(output_1, 2)
#define VOL1   PACK(output_1, 4)

#define FSM    PACK(output_2, 1)
#define PL1    PACK(output_2, 2)
#define VOL2   PACK(output_2, 4)

#define PON    PACK(output_3, 1)
#define NHC    PACK(output_3, 2)
#define VOL3   PACK(output_3, 4)

#define EED    PACK(output_4, 1)
#define CSW    PACK(output_4, 2)
#define HSM    PACK(output_4, 4)

#define EECS   PACK(output_5, 1)
#define PSW    PACK(output_5, 2)
#define EAR    PACK(output_5, 4)

#define NPPC   PACK(output_6, 1)
#define TSE    PACK(output_6, 2)
#define SEL    PACK(output_6, 4)

#define NACS   PACK(output_7, 1)
#define SWE    PACK(output_7, 2)
#define TXS    PACK(output_7, 4)

!----------------------------------------------------------------------

#define swereg output_7
#define iSWE 2

!----------------------------------------------------------------------

#define MIC_ON  SET(HSM)
#define MIC_OFF CLR(HSM)

#define TXS_ON  SET(TXS)
#define TXS_OFF CLR(TXS)

#define WDRESET SET(PON)

!----------------------------------------------------------------------


	.org 0x8000
ram:

outputs:			! Address bits 3..1 special !

output_0:	.rs 2	! every another byte padding, sometimes used as tmp.
output_1:	.rs 2
output_2:	.rs 2
output_3:	.rs 2
output_4:	.rs 2
output_5:	.rs 2
output_6:	.rs 2
output_7:	.rs 2

	.org 0x801F
mdm_data	BYTE	; ASSERT((mdm_data & 0x20) == 0)
mdm_ctrl    BYTE	; ASSERT((mdm_ctrl & 0x20) <> 0)
mdm_stat	BYTE	; ASSERT((mdm_stat & 0x20) <> 0)

	.align 1
asy_data	BYTE	; ASSERT((asy_data & 0x01) == 0)
asy_ctrl	BYTE	! 1
asy_dummy	BYTE	! 0
asy_stat	BYTE	! 1

!----------------------------------------------------------------------
!
!  Inputs registers are canonized into generic (virtual brrh) bits
!

POWEROFF  = 0b00000001	! powerswitch in off-position. Off == 1
TOFF      = 0b00000010	! TX says no output power. TX on == 0 ?
OFFHOOK   = 0b00000100	! handset lifted. Lifted == 1
LOWBATT   = 0b00001000	! under the VCS-selected limit, Under = 1
AUXPTT    = 0b00010000	! aux-ptt active (depending on model). Ptt == 1
LOCAL     = 0b00100000	! service jumper. Local == 1
PORTABLE  = 0b01000000	! battery pack operation. Portable == 1
COLDSTART = 0b10000000	! was powered down over 5 seconds. Cold == 1

input_bits	BYTE

ad_txi 		BYTE
ad_rssi		BYTE
ad_batt		BYTE
ad_temp		BYTE

!----------------------------------------------------------------------
!
!  Channelization

!
!  Lowest channel  is 400.000.0
!  Highest channel is 499.975.0
!  Spacing set to 25kHz
!

#define f2c(f) (((f) - 400000) / 25)

PRESCALER = 80
RX_OFFSET = (21400 / 25)

BASEDIV_N = (400000) / 25 / PRESCALER
BASEDIV_A = (400000) / 25 % PRESCALER

!----------------------------------------------------------------------

			ASSERT(HI(.) == HI(ram))

!----------------------------------------------------------------------

STACK = 0x83FF

!----------------------------------------------------------------------
	.text
	.org 0

	dis
	.byte 0x00

interrupt_XXX:

	req

	LDREG(Rpgm, 1f)

	sep Rpgm
1:
	LDREG(Rint, interrupt_XXX)
	LDREG(Rstk, STACK)
	LDREG(Rbp,  STACK)
	LDREG(Rcal, calsub)
	LDREG(Rret, retsub)
	LDREG(Rram, ram)
	LDREG(Rptr, ram)

	sex Rram

	WDRESET

	ADJUST(Rram, outputs)

#define X(bits) ldi (bits) ; str Rram ; out REG ; inc Rram

	X(0b001)			! 0: RSE      XM       RXE
	X(0b100)			! 1: VOL1     FIBEEP   SV
	X(0b111)			! 2: VOL2     PL1      FSM
	X(0b111)			! 3: VOL3     /HC      PON
	X(0b000)			! 4: S        CSW      D
	X(0b100)			! 5: H        PSW      PCS
	X(0b000)			! 6: SEL      TSE      /PPC
	X(0b001)			! 7: TXS      SWE      /ACS

#undef X

	sex Rpgm
#if XXX
	ret
#else
	dis
#endif
	.byte Rpgm

	WDRESET

	LDREG(Rtmp, 0)
1:
	dec Rtmp
	glo Rtmp
	lbnz 1b
	WDRESET
	ghi Rtmp
	lbnz 1b				! longer delay before touching synthesizer

	CALL(input_register)	! Flush

	WDRESET

	CALL(rxsynth)
	CALL(rxsynth)

	WDRESET

	CALL(txoff)

	lbr top

1:
	sep Rpgm
calsub:
	glo Rbp
	dec Rstk
	str Rstk

	glo Rstk
	plo Rbp
	inc Rbp

	ghi Rarg
	dec Rstk
	str Rstk
	glo Rarg
	dec Rstk
	str Rstk

	ghi Rpgm
	phi Rarg
	glo Rpgm
	plo Rarg

	lda Rarg
	plo Rpgm
	lda Rarg
	phi Rpgm

	br 1b

1:
	sep Rpgm
retsub:
	ghi Rarg
	phi Rpgm
	glo Rarg
	plo Rpgm

	lda Rstk
	plo Rarg
	lda Rstk
	phi Rarg

	lda Rstk
	plo Rbp

	br 1b

!----------------------------------------------------------------------
!
!  Loop here until auxiliary ptt is released
!
wait_aux_ptt:
1:
	WDRESET

	CALL(input_register)

	glo Rtmp
	ani AUXPTT
	bnz 1b

	RETURN

	.align 8

!----------------------------------------------------------------------
!
!  Main commutator start
!
top:
	WDRESET

	!
	!  Look into input shift register
	!

	CALL(input_register)

	glo Rtmp
	ani POWEROFF
	bz 1f
	CLR(PON)
	br .		! Stay
1:
	glo Rtmp
	ani AUXPTT
	bz 1f
	CALL(txon)
	CALL(wait_aux_ptt)
	CALL(txoff)
	br top
1:
	CALL(do_squelch)
	br top

!----------------------------------------------------------------------

txon:
	WDRESET

	CLR(EAR)
	CLR(RXE)
	TXS_ON
	MIC_ON

	CALL(txsynth)

	SET(XM)

	RETURN

txoff:
	WDRESET

	CLR(XM)
	TXS_OFF
	SET(RXE)
	MIC_OFF

	CALL(rxsynth)

	RETURN

!----------------------------------------------------------------------

	.align 8

txsynth:

	ldi LO(f2c(433400))
	plo Rtmp3
	ldi HI(f2c(433400))
	phi Rtmp3

	lbr xxsynth_continue

rxsynth:

	ldi LO(f2c(433400))
	plo Rtmp3
	ldi HI(f2c(433400))
	phi Rtmp3

	DOADD(Rtmp3, RX_OFFSET)

xxsynth_continue:

	WDRESET

	!
	!  Calculate N & A to Rtmp2 & Rtmp3, channel now in Rtmp3
	!  Max 50 loops from 4000 channels divided by 80
	!
	LDREG(Rtmp2, 0xFFFF)	! start from -1 for nice loop.
1:
	inc Rtmp2
	DOSUB(Rtmp3, PRESCALER)
	bdf 1b
	DOADD(Rtmp3, PRESCALER)	! Now ChannelN in Rtmp2, ChannelA in Rtmp3

	glo Rtmp3
	adi BASEDIV_A	! Add BaseA of 400.000MHz
	plo Rtmp3
	smi PRESCALER
	bnf 1f			! go if final A did not overflow
	plo Rtmp3		! Fix % 80, DF set
1:
	glo Rtmp2
	adci LO(BASEDIV_N)
	plo Rtmp2		! Total N from BaseN + ChannelN plus possible carry
	ghi Rtmp2
	adci HI(BASEDIV_N)
	phi Rtmp2		! Need 9 bits to store max N of 260, MSb 0 always



	dec Rstk		! scratch to "write" to

	sex Rpgm
	dis
	.byte Rpgm


	sex Rram

	ADJUST(Rram, output_7)
	lda Rram
	ori 2			! SWE in mirror
	str Rram

	ADJUST(Rram, output_0)
	lda Rram
	ori 4			! RSE in mirror
	str Rram
	
	ADJUST(Rram, output_6)
	lda Rram
	ori 2			! TSE in mirror
	str Rram


	ADJUST(Rram, output_7 + 1)
	out REG			! SWE


	! 512 is 25kHz spacing, binary 0000'0010'0000'0000' (14 bits, really)

	ldi HI(512)
	xri 0xFF
	str Rstk	! 15
	shl
	str Rstk	! 14
	shl
	str Rstk	! 13
	shl
	str Rstk	! 12
	shl
	str Rstk	! 11
	shl
	str Rstk	! 10
	shl
	str Rstk	! 9
	shl
	str Rstk	! 8

	ldi LO(512)
	xri 0xFF
	str Rstk	! 7
	shl
	str Rstk	! 6
	shl
	str Rstk	! 5
	shl
	str Rstk	! 4
	shl
	str Rstk	! 3
	shl
	str Rstk	! 2
	shl
	str Rstk	! 1
	shl
	str Rstk	! 0

	ldi ~0xFF	! Control bit 1 for reference divider.
	str Rstk

	ADJUST(Rram, output_0 + 1)
	out REG		! RSE
	dec Rram
	dec Rram
	out REG		! RSE down

	ADJUST(Rram, output_6 + 1)
	out REG		! TSE
	dec Rram
	dec Rram
	out REG		! TSE down


	ghi Rtmp2
	xri 0xFF		! ------98 -
	shrc			! -------9 8
	shrc			! 8------- 9
	shrc			! 98------ -
	str Rstk	! N9
	shl
	str Rstk	! N8

	glo Rtmp2
	xri 0xFF
	str Rstk	! N7
	shl
	str Rstk	! N6
	shl
	str Rstk	! N5
	shl
	str Rstk	! N4
	shl
	str Rstk	! N3
	shl
	str Rstk	! N2
	shl
	str Rstk	! N1
	shl
	str Rstk	! N0

	glo Rtmp3
	xri 0xFF
	shl
	str Rstk	! A6
	shl
	str Rstk	! A5
	shl
	str Rstk	! A4
	shl
	str Rstk	! A3
	shl
	str Rstk	! A2
	shl
	str Rstk	! A1
	shl
	str Rstk	! A0

	ldi ~0x00
	str Rstk	! Control bit 0 for NA load

	ADJUST(Rram, output_0 + 1)
	out REG		! RSE
	dec Rram
	dec Rram
	out REG		! RSE down

	ADJUST(Rram, output_6 + 1)
	out REG		! TSE
	dec Rram
	dec Rram
	out REG		! TSE down


	ADJUST(Rram, output_7)
	out REG		! SWE down

	sex Rpgm
#if XXX
	ret
#else
	dis
#endif
	.byte Rpgm

	inc Rstk

	WDRESET

	RETURN

	.align 8

!----------------------------------------------------------------------

input_register:

	WDRESET

	ADJUST(Rram, mdm_stat)
	sex Rram
	inp MDM			! Read status register

	ADJUST(Rram, asy_stat)
	sex Rram
	inp ASY

	SET(EED)		! ADC DI
	req				! ADC CLK
	nop

	WDRESET

	CLR(NACS)		! ADC /CS
	nop

	seq; nop; nop; req; nop; nop	! Start bit "1"
	seq; nop; nop; req; nop; nop	! Single-ended "1"
	seq; nop; nop; req; nop; nop	! /ODD is "1"

	CLR(EED)
	nop; nop

	seq; nop; nop; req; nop; nop	! "0" of 01

	SET(EED)
	nop; nop

	seq; nop; nop; req; nop; nop	! "1" of 01
	seq; nop; nop; req;				! MUX settling bit

	WDRESET

	ADJUST(Rram, ad_rssi)
	ldi 0

	seq; b4 1f; ori 0x80; 1: nop
	seq; b4 1f; ori 0x40; 1: nop
	seq; b4 1f; ori 0x20; 1: nop
	seq; b4 1f; ori 0x10; 1: nop

	seq; b4 1f; ori 0x08; 1: nop
	seq; b4 1f; ori 0x04; 1: nop
	seq; b4 1f; ori 0x02; 1: nop
	seq; b4 1f; ori 0x01; 1:

	str Rram

	SET(NACS)		! ADC /CS inactive

	WDRESET

	ADJUST(Rram, input_bits)
	ldi 0

	bn2 1f
	ori AUXPTT
1:
	bn1 1f
	ori POWEROFF
1:

	plo Rtmp
	sex Rram
	sm
	lbz 1f
	glo Rtmp
	str Rram
1:
	RETURN

do_squelch:

	WDRESET

	ADJUST(Rram, ad_rssi)

	ldn Rram
	smi 50
	bdf 1f
	CLR(EAR)
	RETURN
1:
	SET(EAR)
	RETURN

